"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MessageList = void 0;
var _Observable = require("./Observable");
var _types = require("../types");
var _utils = require("../utils");
class MessageList extends _Observable.Observable {
  constructor() {
    let initialData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    super();
    this.messages = initialData;
  }
  addMessage(incomingMessage) {
    const messages = this.messages.filter(msg => msg.type !== _types.MessageTypes.TYPING);
    const message = {
      ...incomingMessage,
      hour: (0, _utils.getHourAndMinutes)()
    };
    messages.push(message);
    this.messages = messages;
    this.notify('addMessage', message);
  }
  changeStatus(id, status) {
    this.messages = this.messages.map(msg => {
      if (msg.id === id) return {
        ...msg,
        status
      };
      return msg;
    });
  }
  clearMessages() {
    this.messages = [];
    this.notify('clearMessages', null);
  }
}
exports.MessageList = MessageList;
//# sourceMappingURL=MessageList.js.map