import React, { useEffect, useState } from 'react';
import SessionContextProvider from '../context/SessionContext';
import { Session } from '../entities/Session';
import { Observer } from '../entities/Observer';
export const SessionProvider = _ref => {
  let {
    sessionGateway,
    children
  } = _ref;
  const [initialSession, setInitialSession] = useState(new Session(''));
  useEffect(() => {
    const getInitialSession = async () => {
      const sessionId = await sessionGateway.getCurrent();
      const session = await sessionGateway.subscribe(sessionId);
      sessionGateway.onEndConversation(async () => {
        setInitialSession(await sessionGateway.clearSession());
      });
      session.register(new Observer('sendAction', async data => {
        await sessionGateway.sendAction(data);
      }));
      session.register(new Observer('clearSession', async () => {
        await sessionGateway.clearSession();
        const newSession = await sessionGateway.subscribe('');
        newSession.observers = session.observers;
        setInitialSession(newSession);
      }));
      setInitialSession(session);
    };
    getInitialSession();
  }, []);
  return /*#__PURE__*/React.createElement(SessionContextProvider, {
    initialSession: initialSession
  }, children);
};
//# sourceMappingURL=SessionProvider.js.map